@@ -6,8 +6,13 @@ package Alien::FLTK;
     use File::Spec::Functions qw[catdir rel2abs canonpath];
     use File::Basename;
     use File::Find qw[find];
-    our $VERSION_BASE = 0; our $FLTK_SVN = 6841; our $UNSTABLE_RELEASE = 0; our $VERSION = sprintf('%d.%05d' . ($UNSTABLE_RELEASE ? '_%03d' : ''), $VERSION_BASE, $FLTK_SVN, $UNSTABLE_RELEASE);
+    our $VERSION_BASE = 0; our $FLTK_SVN = 6844; our $UNSTABLE_RELEASE = 0; our $VERSION = sprintf('%d.%05d' . ($UNSTABLE_RELEASE ? '_%03d' : ''), $VERSION_BASE, $FLTK_SVN, $UNSTABLE_RELEASE);
     sub revision { return $FLTK_SVN; }
+    my $_config = eval do { local $/; <DATA> }
+        or warn
+        "Couldn't load Alien::FLTK configuration data: $@\n Using defaults";
+    close DATA;
+    sub config { return $_config; }
 
     sub include_path {
         my @include = map { -d $_ ? $_ : () } (
@@ -26,103 +31,50 @@ package Alien::FLTK;
     }
 
     sub cflags {
-        my $CFLAGS = '-I' . Alien::FLTK->include_path();
-        if (($Config{'osname'} || $^O) =~ m[MSWin32]) {
-            $CFLAGS = "$CFLAGS -mwindows -DWIN32";
-        }
-        return $CFLAGS;
+        return $_config->{'cxxflags'} . ' -I' . Alien::FLTK->include_path();
     }
+    sub cxxflags { return Alien::FLTK->cflags() . ' -Wno-non-virtual-dtor'; }
 
-    sub cxxflags {
-        my $CXXFLAGS = Alien::FLTK->cflags();
-        if (($Config{'osname'} || $^O) =~ m[MSWin32]) {
-            $CXXFLAGS = "$CXXFLAGS -Wno-non-virtual-dtor";
-        }
-        return $CXXFLAGS;
-    }
-
-    sub ldflags {
+    sub ldflags {    # XXX - Cache this
         my ($self, @args) = @_;
-        my $LDLIBS = my $GLLIB = '';
-        {
-            local $_ = ($Config{'osname'} || $^O);
-            if (m[MSWin32]) {
-                $LDLIBS
-                    = '-mwindows -lmsimg32 -lole32 -luuid -lcomctl32 -lwsock32 -lsupc++';
-                $GLLIB = "-lopengl32"
-                    if _find_h('GL/gl.h');    # XXX only if use_gl
-                $GLLIB = "-lglu32 $GLLIB" if _find_h('GL/glu.h');
-            }
-            elsif (m[MacOS]) {    # MacOS X uses Carbon for graphics...
-                $LDLIBS = '-framework Carbon -framework ApplicationServices';
-                $GLLIB  = '-framework AGL -framework OpenGL';
-            }
-            else {                # All others are UNIX/X11...
-                $LDLIBS
-                    = '-lX11 -lXi -lXcursor -lpthread -lm -lXext -lsupc++';
-                if (_find_h('GL/gl.h')) {
-                    $GLLIB = _find_lib('MesaGL') ? '-lMesaGL' : '-lGL';
-                    if (_find_h('GL/glu.h')) {
-                        $GLLIB = "-lGLU $GLLIB"     if _find_lib('GLU');
-                        $GLLIB = "-lMesaGLU $GLLIB" if _find_lib('MesaGL');
-                    }
-                }
-            }
-        }
 
         #
         my $libdir = Alien::FLTK->library_path();
 
         # Calculate needed libraries
         my $SHAREDSUFFIX = $Config{'_a'};
-        my $LDSTATIC     = "-L$libdir $libdir/libfltk2$SHAREDSUFFIX $LDLIBS";
-        my $LDFLAGS      = "-L$libdir -lfltk2 $LDLIBS";
-        my $LIBS         = "$libdir/libfltk2$SHAREDSUFFIX";
-        my $IMAGELIBS = " -lfltk2_png -lfltk2_z -lfltk2_images -lfltk2_jpeg ";
+        my $LDSTATIC     = "-L$libdir $libdir/libfltk2$SHAREDSUFFIX "
+            . $_config->{'ldflags'};
+        my $LDFLAGS = "-L$libdir " . $_config->{'ldflags'};
+        my $LIBS    = "$libdir/libfltk2$SHAREDSUFFIX";
         if (grep {m[forms]} @args) {
             $LDFLAGS  = "-lfltk2_forms $LDFLAGS";
             $LDSTATIC = "$libdir/libfltk2_forms$SHAREDSUFFIX $LDSTATIC";
             $LIBS     = "$LIBS $libdir/libfltk2_forms$SHAREDSUFFIX";
         }
-        if (grep {m[gl]} @args) {
-            $LDFLAGS  = "-lfltk2_gl $GLLIB $LDFLAGS";
-            $LDSTATIC = "$libdir/libfltk2_gl$SHAREDSUFFIX $GLLIB $LDSTATIC";
+        if ((grep {m[gl]} @args) && $_config->{'GL'}) {
+            my $LIBGL = $_config->{'GL'};
+            $LDFLAGS  = "-lfltk2_gl $LIBGL $LDFLAGS";
+            $LDSTATIC = "$libdir/libfltk2_gl$SHAREDSUFFIX $LIBGL $LDSTATIC";
             $LIBS     = "$LIBS $libdir/libfltk2_gl$SHAREDSUFFIX";
         }
         if (grep {m[images]} @args) {
-            $LDFLAGS = "-lfltk2_images $IMAGELIBS $LDFLAGS";
-            $LDSTATIC
-                = "$libdir/libfltk2_images$SHAREDSUFFIX $LDSTATIC $IMAGELIBS";
+            $LDFLAGS  = $_config->{'image_flags'} . " $LDFLAGS";
+            $LDSTATIC = "$libdir/libfltk2_images$SHAREDSUFFIX $LDSTATIC "
+                . $_config->{'image_flags'};
         }
-        return ((grep {m[static]} @args) ? $LDSTATIC : $LDFLAGS);
+        return (
+             ((grep {m[static]} @args) ? $LDSTATIC : $LDFLAGS) . ' -lsupc++');
     }
 
-    sub _find_lib {
-        my ($find) = @_;
-        $find =~ s[([\+\*\.])][\\$1]g;
-        my $lib;
-        find(
-            sub {
-                $lib = $File::Find::name
-                    if $_ =~ qr[lib$find$Config{'_a'}];
-            },
-            split ' ',
-            $Config{'libpth'}
-        );
-        return $lib;
-    }
+    sub capabilities {
+        my @caps;
+        push @caps, 'gl' if $_config->{'config'}{'HAVE_GL'};
 
-    sub _find_h {
-        my $file = rel2abs(catdir($Config{'incpath'}, shift));
-        my $found = 0;
-        find(
-            sub {
-                $found = 1 if canonpath($File::Find::name) eq $file;
-            },
-            $Config{'incpath'}
-        );
-        return $found;
+        # TODO: images, forms, static(?)
+        return @caps;
     }
+    1
 }
 
 =pod
@@ -167,7 +119,7 @@ C<2.0.x> branch of the FLTK GUI toolkit.
                                   objects            => $obj,
                                   extra_linker_flags => Alien::FLTK->ldflags()
     );
-    print system($exe) ? 'Aww...' : 'Yay!';
+    print system('./' . $exe) ? 'Aww...' : 'Yay!';
     END { unlink grep defined, $source, $obj, $exe; }
 
 =head1 Methods
@@ -176,8 +128,7 @@ C<2.0.x> branch of the FLTK GUI toolkit.
 
     my $include_path = Alien::FLTK->include_path;
 
-Returns the location of the headers installed during the build
-process.
+Returns the location of the headers installed during the build process.
 
 =head2 C<library_path>
 
@@ -196,15 +147,14 @@ Returns additional C compiler flags to be used.
 
     my $cxxflags = Alien::FLTK->cxxflags;
 
-Returns additional C++ compiler flags to be used.
-to compile C++ using FLTK
+Returns additional flags to be used to when compiling C++ using FLTK.
 
-=head2 C<cxxflags>
+=head2 C<ldflags>
 
-    my $cxxflags = Alien::FLTK->ldflags(qw[gl images]);
+    my $ldflags = Alien::FLTK->ldflags(qw[gl images]);
 
-Returns additional linker flags to be used. This method can automatically
-add appropriate flags based on how you plan on linking to fltk. Acceptable
+Returns additional linker flags to be used. This method can automatically add
+appropriate flags based on how you plan on linking to fltk. Acceptable
 arguments are:
 
 =over
@@ -242,17 +192,68 @@ Include flags to use FLTK's forms compatibility layer.
 
 =head2 C<revision>
 
-    my $revision = Alien::wxWidgets->revision;
+    my $revision = Alien::FLTK->revision;
 
-Returns the SVN revision number of the source
-L<C<Alien::FLTK>|Alien::FLTK> was built with.
-
-=head1 Bugs
-
-Numerous, I'm sure.
+Returns the SVN revision number of the source L<C<Alien::FLTK>|Alien::FLTK>
+was built with.
 
 =head1 Notes
 
+=head2 Requirements
+
+The Fast Light Toolkit works right out of the box on Windows (MinGW), but for
+*nix, make sure the following libs are installed for basic functionality...
+
+=over
+
+=item X11 libs
+
+On Debian Linux, the X11 development package is listed as libx11-dev.
+
+=item X11 Input libs
+
+On Debian Linux, the X11 Input development package is listed as libxi-dev.
+
+=item X11 Cursor libs
+
+On Debian Linux, the X11 Cursor development package is listed as
+libxcursor-dev.
+
+Note: The Xcursor libs are optional.
+
+=back
+
+Once installed, L<Alien::FLTK> depends on:
+
+=over
+
+=item L<Config|Config>
+
+=item L<File::Spec::Functions>
+
+=item L<File::Basename>
+
+=item L<File::Find>
+
+=back
+
+=head2 Installation
+
+Building the fltk2 libs requires a functioning C++ compiler, bash, and (to
+make life easy) a version of make.
+
+The distribution is based on L<Module::Build|Module::Build>, so use the
+following procedure:
+
+  perl Build.PL
+  ./Build
+  ./Build test
+  ./Build install
+
+An attempt has been made to work around an incomplete set of build tools. This
+fallback requires L<ExtUtils::CBuilder|ExtUtils::CBuilder> and plenty of
+begging. Consider it alpha at best.
+
 =head2 Support Links
 
 =over
@@ -290,41 +291,15 @@ http://github.com/sanko/alien-fltk/ and you are invited to fork it.
 
 =back
 
-=head2 Dependencies
-
-=over
-
-=item L<File::Spec::Functions|File::Spec::Functions>
-
-=item L<Module::Build|Module::Build>
-
-=item L<ExtUtils::CBuilder|ExtUtils::CBuilder>
-
-=back
-
 =head2 Examples
 
 Please see the L<Synopsis|/"Synopsis"> and the files in the C</example/>.
 
-=head2 Installation
+=head2 Bugs
 
-This distribution requires a functioning C++ compiler and (to make life easy)
-a version of make.
+Numerous, I'm sure.
 
-The distribution is based on L<Module::Build|Module::Build> and
-L<ExtUtils::CBuilder|ExtUtils::CBuilder>, so use the following procedure:
-
-  perl Build.PL
-  ./Build
-  ./Build test
-  ./Build install
-
-=head3 Win32
-
-An attempt has been made to work around the lack of proper build tools on
-Windows machines. Consider it alpha at best.
-
-=head1 To Do
+=head2 To Do
 
 Please see L<Alien::FLTK::Todo|Alien::FLTK::Todo>
 
@@ -363,6 +338,8 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 L<C<Alien::FLTK>|Alien::FLTK> is based in part on the work of the FLTK
 project. See http://www.fltk.org/.
 
-=for git $Id: FLTK.pm eac9de5 2009-08-22 05:38:21Z sanko@cpan.org $
+=for git $Id: FLTK.pm 7ea0002 2009-09-03 05:03:50Z sanko@cpan.org $
 
 =cut
+__DATA__
+do{ my $x = { }; $x; }
\ No newline at end of file
